case class PaintState(
  color: String,
  width: Int,
  paths: List[Path],
  clipboard: Option[Path],
  currentPath: Option[Path],
  selected: Int,
  dragStart: (Double,Double)
):
  def isSelected: Boolean = selected > -1 && selected < paths.length
  def modifySelected(f: Path => Path): PaintState = {
    val (before,s :: after) = paths.splitAt(selected)
    copy(paths = before ++ (f(s) :: after))
  }

object PaintState:
  def initial = PaintState("black",5,List.empty,Option.empty,None,-1,(0,0))

enum PaintCommand:
  case ChooseColor(color: String)
  case IncStrokeWidth
  case DecStrokeWidth
  case BeginDrag(x: Double, y: Double)
  case Drag(x: Double, y: Double)
  case EndDrag(x: Double, y: Double)
  case Cut
  case Copy
  case Paste
  case SelectNext
  case SelectPrev